#version 150 compatibility

uniform sampler2D colortex0;

varying vec2 texcoord;

// From https://9to5answer.com/from-rgb-to-hsv-in-opengl-glsl under wtpfl
// All components are in the range [0…1], including hue.
vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

// All components are in the range [0…1], including hue.
vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}
// End code from answer

float stepper(in float x)
{
    int value = int(16 * x + 0.5); //int between 0 and steps+0.5
    float ret = float(value - 0.5) * (1.0/ 16);
    return ret;
}

void main() {
    vec3 color = texture2D(colortex0, texcoord).rgb;
    vec3 hsv = rgb2hsv(color);
    hsv.z = stepper(hsv.z);
    if(hsv.y != 0) { // Make sure that white color is not distortet
		hsv.y = min(hsv.y + 0.2, 1.0);
    }
    color = hsv2rgb(hsv);
    gl_FragData[0] = vec4(color, 1.0);
}
